///////////////////////////////////////////////////////////////////////
// MEMBUG.CPP
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int PASCAL WinMain( HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR     lpszCmdLine,
                    int       nCmdShow )
{
#define ALLOC_TRIES 100
#define ALLOC_SIZE  20000

char          **buffers;
int             bugSel;
long            memBefore, memAfter;
static char            memSfree[250];
int             inx;
const char     *title;

// How much memory do we have before ANY allocations?
memBefore = GetFreeSpace(0);

buffers = new char *[ALLOC_TRIES];
bugSel = MessageBox(NULL,"For 'new char[x]' use erroneous 
'delete x' ?",
                    "delete error tester",
                    MB_YESNO );
// Easy rule to remember, if you use square brackets "[]" with
// the "new" operator, then you MUST also use square brackets "[]"
// with the corresponding "delete"
if (bugSel == IDYES) // Ok, we may crash this puppy, but we'll 
try
  {
  for (inx=0; inx<ALLOC_TRIES; inx++)
    buffers[inx] = new char[ALLOC_SIZE];
  // Now make 2 passes to delete, first even entries
  // then odd, to increase memory fragmentation potential
  for (inx=0; inx<ALLOC_TRIES; inx+=2)
    delete buffers[inx]; // error, should be delete [] buffers[inx];
  for (inx=1; inx<ALLOC_TRIES; inx+=2)
    delete buffers[inx]; // error, should be delete [] buffers[inx];
  delete buffers; // error, should be delete [] buffers;
  }
else
  {
  for (inx=0; inx<ALLOC_TRIES; inx++)
    buffers[inx] = new char[ALLOC_SIZE];
  // Now make 2 passes to delete, first even entries
  // then odd, to increase memory fragmentation potential
  for (inx=0; inx<ALLOC_TRIES; inx+=2)
    delete [] buffers[inx]; // correct
  for (inx=1; inx<ALLOC_TRIES; inx+=2)
    delete [] buffers[inx]; // correct
  delete [] buffers; // correct
  }

// How much memory do we have after all "deletes" ?

memAfter =  GetFreeSpace(0);

// Generate report message
sprintf(memSfree,
"Memory Test Result\r\n\
%11ld bytes before\r\n\
%11ld bytes after\r\n\
%11ld bytes delta",
        memBefore, memAfter, memBefore-memAfter );

if (memBefore == memAfter)  title = "No Data Loss";
else                        title = "ERROR, Lost memory";

MessageBox( NULL, memSfree, title, MB_OK | MB_ICONASTERISK );

return(0);
}
// End of File 

